# Generated by Django 2.2 on 2019-04-15 13:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0018_auto_20190414_0923'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Cdw',
        ),
        migrations.DeleteModel(
            name='Spw',
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-13 13:07

from django.db import migrations

from kaf_pas.ckk.models.attr_type import Attr_type


def rec_lotsman_attr_type(apps, schema_editor):
    KD, _ = Attr_type.objects.update_or_create(code='KD', defaults=dict(name='КД'))
    LOTSMAN, _ = Attr_type.objects.update_or_create(code='LOTSMAN', defaults=dict(parent=KD, name='Документы из Лоцмана'))
    return LOTSMAN


class Migration(migrations.Migration):
    dependencies = [
        ('kd', '0090_auto_20191213_1303'),
    ]

    operations = [
        migrations.RunPython(rec_lotsman_attr_type),
    ]



from django.urls import path

from kaf_pas.kd.views import document_attributes_Name_material

urlpatterns = [

    path('Document_attributes_Name_material/Fetch/', document_attributes_Name_material.Document_attributes_Name_material_Fetch),
    path('Document_attributes_Name_material/Add', document_attributes_Name_material.Document_attributes_Name_material_Add),
    path('Document_attributes_Name_material/Update', document_attributes_Name_material.Document_attributes_Name_material_Update),
    path('Document_attributes_Name_material/Remove', document_attributes_Name_material.Document_attributes_Name_material_Remove),
    path('Document_attributes_Name_material/Lookup/', document_attributes_Name_material.Document_attributes_Name_material_Lookup),
    path('Document_attributes_Name_material/Info/', document_attributes_Name_material.Document_attributes_Name_material_Info),
    path('Document_attributes_Name_material/Copy', document_attributes_Name_material.Document_attributes_Name_material_Copy),

]



# Generated by Django 2.2.1 on 2019-05-21 23:04

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0053_auto_20190521_1939'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='status',
            name='id_owner',
        ),
        migrations.AlterField(
            model_name='status',
            name='code_owner',
            field=isc_common.fields.code_field.CodeField(unique=True),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-22 22:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0059_auto_20190821_0950'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operation_material',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='operation_plan_types',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='operation_resources',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='operations',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='operations_item',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='operationsplan',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='resource',
            name='id_old',
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-19 19:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0003_auto_20190918_1437'),
    ]

    operations = [
        migrations.AddField(
            model_name='operationsplan',
            name='quantity_minus',
            field=models.DecimalField(decimal_places=8, default=0, max_digits=20),
        ),
        migrations.AddField(
            model_name='operationsplan',
            name='quantity_plus',
            field=models.DecimalField(decimal_places=8, default=0, max_digits=20),
        ),
        migrations.AlterModelTable(
            name='operations_plan_view',
            table='planing_operationsplan_view',
        ),
        migrations.DeleteModel(
            name='Operations_plan_qtys',
        ),
    ]



# Generated by Django 3.0.2 on 2020-01-18 15:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0061_auto_20200118_1439'),
    ]

    operations = [
        migrations.AddField(
            model_name='operations',
            name='description',
            field=models.TextField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-28 19:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0075_remove_pathes_attr_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='pathes',
            name='virt_path',
            field=models.TextField(blank=True, null=True, verbose_name='Мнимый путь'),
        ),
    ]



import logging

from bitfield import BitField
from django.db.models import UniqueConstraint, Q

from clndr.models.calendars import Calendars
from isc_common.fields.related import ForeignKeyProtect
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsQuerySet, CommonManagetWithLookUpFieldsManager
from isc_common.models.base_ref import BaseRefQuerySet, BaseRef
from isc_common.number import DelProps
from kaf_pas.ckk.models.locations import Locations

logger = logging.getLogger(__name__)


class ResourceQuerySet(CommonManagetWithLookUpFieldsQuerySet, BaseRefQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class ResourceManager(CommonManagetWithLookUpFieldsManager):
    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'description': record.description,
            'calendar_id': record.calendar.id if record.calendar else None,
            'calendar__full_name': record.calendar.full_name if record.calendar else None,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return ResourceQuerySet(self.model, using=self._db)


class Resource(BaseRef):
    location = ForeignKeyProtect(Locations)
    calendar = ForeignKeyProtect(Calendars, null=True, blank=True)

    objects = ResourceManager()

    @property
    def get_calendar(self):
        return self.calendar

    def __str__(self):
        return f"ID: {self.id}, code: {self.code}, name: {self.name}, description: {self.description}, location: [{self.location}]"

    def __repr__(self):
        return self.name

    class Meta:
        verbose_name = 'Ресурсы'
        constraints = [
            UniqueConstraint(fields=['location'], condition=Q(code=None) & Q(name=None), name='Resource_unique_constraint_0'),
            UniqueConstraint(fields=['location', 'name'], condition=Q(code=None), name='Resource_unique_constraint_1'),
            UniqueConstraint(fields=['code', 'location'], condition=Q(name=None), name='Resource_unique_constraint_2'),
            UniqueConstraint(fields=['code', 'location', 'name'], name='Resource_unique_constraint_3'),
        ]



# Generated by Django 2.1.7 on 2019-03-27 11:41

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0003_auto_20190327_1111'),
    ]

    operations = [
        migrations.AddField(
            model_name='item',
            name='props',
            field=bitfield.models.BitField((('relevant', 'Актуальность'),), default=1),
        ),
        migrations.AddField(
            model_name='item_line',
            name='props',
            field=bitfield.models.BitField((('relevant', 'Актуальность'), ('relevant1', 'Актуальность1')), default=1),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-26 11:27

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0147_auto_20190822_2250'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item_line',
            name='child',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='item_child', to='ckk.Item', verbose_name='Товарная позиция'),
        ),
        migrations.AlterField(
            model_name='item_refs',
            name='child',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='child', to='ckk.Item'),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-25 09:53

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0060_auto_20190524_1536'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item_image_refs',
            name='item',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='ckk.Item'),
        ),
    ]



# Generated by Django 2.1.7 on 2019-03-28 07:05

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0003_auto_20190327_1141'),
        ('ckk', '0005_auto_20190327_2032'),
    ]

    operations = [
        migrations.AddField(
            model_name='item',
            name='document',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='kd.Documents', verbose_name='Документ'),
        ),
    ]



from django.urls import path

from kaf_pas.production.views import launch_item_line

urlpatterns = [

    path('Launch_item_line/Fetch/', launch_item_line.Launch_item_line_Fetch),
    path('Launch_item_line/Add', launch_item_line.Launch_item_line_Add),
    path('Launch_item_line/Update', launch_item_line.Launch_item_line_Update),
    path('Launch_item_line/Remove', launch_item_line.Launch_item_line_Remove),
    path('Launch_item_line/Lookup/', launch_item_line.Launch_item_line_Lookup),
    path('Launch_item_line/Info/', launch_item_line.Launch_item_line_Info),
    path('Launch_item_line/Copy', launch_item_line.Launch_item_line_Copy),

]



# Generated by Django 2.2.2 on 2019-07-01 17:12

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0116_auto_20190701_1705'),
    ]

    operations = [
        migrations.AddField(
            model_name='materials',
            name='type',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='ckk.Material_type'),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-12 18:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0169_item_refs_syntetic_child'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='item_refs',
            unique_together={('child', 'parent', 'syntetic_child')},
        ),
    ]



import logging

from django.core.management import BaseCommand

from kaf_pas.planing.models.levels import Levels
from kaf_pas.planing.models.routing import RoutingManager

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    def handle(self, *args, **options):
        for level in Levels.objects.all():
            print('\n\n')
            for a in RoutingManager.make_resourcesLevel(launch_id=30, level_id=16, location_id=64):
                print(a)



# Generated by Django 3.0.3 on 2020-02-17 13:55

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0192_auto_20200217_1221'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_operations_item',
            name='operationitem',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='production.Operations_item'),
        ),
    ]



# Generated by Django 3.0.3 on 2020-03-02 08:08

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0123_auto_20200301_1424'),
    ]

    operations = [
        migrations.AddField(
            model_name='operation_value',
            name='props',
            field=bitfield.models.BitField((('perone', 'Из расчета на еденицу продукции'),), db_index=True, default=0),
        ),
    ]



# Generated by Django 2.2 on 2019-04-14 07:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0010_auto_20190413_0700'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attr_type',
            name='deleted',
            field=models.BooleanField(blank=True, db_index=True, null=True, verbose_name='Признак удаления'),
        ),
        migrations.AlterField(
            model_name='item',
            name='deleted',
            field=models.BooleanField(blank=True, db_index=True, null=True, verbose_name='Признак удаления'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='deleted',
            field=models.BooleanField(blank=True, db_index=True, null=True, verbose_name='Признак удаления'),
        ),
        migrations.AlterField(
            model_name='item_line_refs',
            name='deleted',
            field=models.BooleanField(blank=True, db_index=True, null=True, verbose_name='Признак удаления'),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-10 10:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0087_auto_20190610_0953'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='materials',
            name='code',
        ),
        migrations.RemoveField(
            model_name='materials',
            name='description',
        ),
        migrations.RemoveField(
            model_name='materials',
            name='name',
        ),
    ]



# Generated by Django 3.0.2 on 2020-01-18 14:39

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0060_auto_20200118_1436'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operations',
            name='status',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='planing.Status_operation_types'),
        ),
    ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.input.models.users_ex import Users_ex, Users_exManager


@JsonResponseWithException()
def Users_ex_Fetch(request):
    from isc_common.auth.models.user import User
    return JsonResponse(
        DSResponse(
            request=request,
            data=Users_ex.objects.
                filter().
                exclude(props=User.props.bot).
                get_range_rows1(
                request=request,
                function=Users_exManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Users_ex_Add(request):
    return JsonResponse(DSResponseAdd(data=Users_ex.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Users_ex_Update(request):
    return JsonResponse(DSResponseUpdate(data=Users_ex.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Users_ex_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Users_ex.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Users_ex_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Users_ex.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Users_ex_Info(request):
    return JsonResponse(DSResponse(request=request, data=Users_ex.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Users_ex_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Users_ex.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.ckk.models.item_refs_hierarcy import Item_refs_hierarcyManager, Item_refs_hierarcy


@JsonResponseWithException()
def Item_refs_hierarcy_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Item_refs_hierarcy.objects.
                filter().
                get_range_rows1(
                request=request,
                function=Item_refs_hierarcyManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_refs_hierarcy_Add(request):
    return JsonResponse(DSResponseAdd(data=Item_refs_hierarcy.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_refs_hierarcy_Update(request):
    return JsonResponse(DSResponseUpdate(data=Item_refs_hierarcy.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_refs_hierarcy_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Item_refs_hierarcy.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_refs_hierarcy_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Item_refs_hierarcy.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_refs_hierarcy_Info(request):
    return JsonResponse(DSResponse(request=request, data=Item_refs_hierarcy.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_refs_hierarcy_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Item_refs_hierarcy.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



import logging

from django.db.models import CheckConstraint, Q, F, UniqueConstraint

from isc_common.fields.code_field import CodeField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from isc_common.models.tree_audit import TreeAuditModelManager
from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy

logger = logging.getLogger(__name__)


class Lotsman_documents_hierarcy_refsQuerySet(AuditQuerySet):
    pass

class Lotsman_documents_hierarcy_refsManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Lotsman_documents_hierarcy_refsQuerySet(self.model, using=self._db)


class Lotsman_documents_hierarcy_refs(AuditModel):
    child = ForeignKeyProtect(Lotsman_documents_hierarcy, related_name='Lotsman_documents_hierarcy_child')
    parent = ForeignKeyProtect(Lotsman_documents_hierarcy, related_name='Lotsman_documents_hierarcy_parent', null=True, blank=True)

    section = CodeField(verbose_name="Раздел", null=True, blank=True)
    subsection = CodeField(verbose_name="Подраздел", null=True, blank=True)

    objects = Lotsman_documents_hierarcy_refsManager()
    objects_tree = TreeAuditModelManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Кросс таблица'
        constraints = [
            UniqueConstraint(fields=['child'], condition=Q(parent=None) & Q(section=None) & Q(subsection=None), name='Lotsman_documents_hierarcy_refs_unique_constraint_0'),
            UniqueConstraint(fields=['child', 'parent'], condition=Q(section=None) & Q(subsection=None), name='Lotsman_documents_hierarcy_refs_unique_constraint_1'),
            UniqueConstraint(fields=['child', 'section'], condition=Q(parent=None) & Q(subsection=None), name='Lotsman_documents_hierarcy_refs_unique_constraint_2'),
            UniqueConstraint(fields=['child', 'parent', 'section'], condition=Q(subsection=None), name='Lotsman_documents_hierarcy_refs_unique_constraint_3'),
            UniqueConstraint(fields=['child', 'subsection'], condition=Q(parent=None) & Q(section=None), name='Lotsman_documents_hierarcy_refs_unique_constraint_4'),
            UniqueConstraint(fields=['child', 'parent', 'subsection'], condition=Q(section=None), name='Lotsman_documents_hierarcy_refs_unique_constraint_5'),
            UniqueConstraint(fields=['child', 'section', 'subsection'], condition=Q(parent=None), name='Lotsman_documents_hierarcy_refs_unique_constraint_6'),
            UniqueConstraint(fields=['child', 'parent', 'section', 'subsection'], name='Lotsman_documents_hierarcy_refs_unique_constraint_7'),
            CheckConstraint(check=~Q(child=F('parent')), name='Lotsman_documents_hierarcy_not_circulate_refs')
        ]



import logging

from django.db.models import BigAutoField, DateTimeField, BooleanField, Model
from django.utils import timezone

from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade
from isc_common.models.audit import AuditManager, AuditQuerySet
from kaf_pas.kd.models.document_attributes import Document_attributes
from kaf_pas.kd.models.documents import Documents
from kaf_pas.kd.models.uploades import Uploades

logger = logging.getLogger(__name__)


class Uploades_documentsViewQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Uploades_documentsViewManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'document_id': record.document.id,
            'document__path_id': record.document.path.id,
            'STMP_1_id': record.STMP_1.id if record.STMP_1 else None,
            'STMP_1__value_str': record.STMP_1.value_str if record.STMP_1 else None,
            'STMP_2_id': record.STMP_2.id if record.STMP_2 else None,
            'STMP_2__value_str': record.STMP_2.value_str if record.STMP_2 else None,
            'document__attr_type_id': record.document.attr_type.id,
            'document__attr_type__code': record.document.attr_type.code,
            'document__attr_type__name': record.document.attr_type.name,
            'document__file_document': record.document.file_document,
            'document__file_size': record.document.file_size,
            'document__lastmodified': record.document.lastmodified,
            'document__file_modification_time': record.document.file_modification_time,
            'document__file_access_time': record.document.file_access_time,
            'document__file_change_time': record.document.file_change_time,
            'editing': record.editing,
            'deliting': record.deliting,
            'isLotsman': record.isLotsman,
        }
        return res

    def get_queryset(self):
        return Uploades_documentsViewQuerySet(self.model, using=self._db)


class Uploades_documents_view(Model):
    id = BigAutoField(primary_key=True, verbose_name="Идентификатор")
    deleted_at = DateTimeField(verbose_name="Дата мягкого удаления", null=True, blank=True, db_index=True)
    editing = BooleanField(verbose_name="Возможность редактирования", default=True)
    deliting = BooleanField(verbose_name="Возможность удаления", default=True)
    lastmodified = DateTimeField(verbose_name='Последнее обновление', editable=False, db_index=True, default=timezone.now)
    isLotsman = BooleanField()

    STMP_1 = ForeignKeyProtect(Document_attributes, verbose_name='Наименование изделия', related_name='STMP_1_upload_doc_view', null=True, blank=True)
    STMP_2 = ForeignKeyProtect(Document_attributes, verbose_name='Обозначение изделия', related_name='STMP_2_upload_doc_view', null=True, blank=True)

    upload = ForeignKeyCascade(Uploades)
    document = ForeignKeyCascade(Documents)

    objects = Uploades_documentsViewManager()

    def __str__(self):
        return f'{self.id}'

    class Meta:
        db_table = 'kd_uploades_documents_view'
        managed = False



# Generated by Django 3.0.3 on 2020-02-28 12:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0220_auto_20200228_1211'),
    ]

    operations = [
        migrations.AddConstraint(
            model_name='resource',
            constraint=models.UniqueConstraint(condition=models.Q(('code', None), ('name', None)), fields=('location',), name='Resource_unique_constraint_0'),
        ),
        # migrations.AddConstraint(
        #     model_name='resource',
        #     constraint=models.UniqueConstraint(condition=models.Q(code=None), fields=('location', 'name'), name='Resource_unique_constraint_1'),
        # ),
    ]



import logging

from django.db.models import DateTimeField, BigIntegerField, TextField, BooleanField

from isc_common.bit import IsBitOff
from isc_common.fields.code_field import CodeField
from isc_common.fields.description_field import DescriptionField
from isc_common.fields.name_field import NameField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from kaf_pas.sales.models.precent_item_types import Precent_item_types
from kaf_pas.sales.models.precent_types import Precent_types
from kaf_pas.sales.models.status_precent import StatusPrecent

logger = logging.getLogger(__name__)


class Precent_viewQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Precent_viewManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'date': record.date,
            'date_sign': record.date_sign,
            'description': record.description,

            'status_id': record.status.id if record.status else None,
            'status__code': record.status.code if record.status else None,
            'status__name': record.status.name if record.status else None,

            'precent_item_type_id': record.precent_item_type.id if record.precent_item_type else None,
            'precent_item_type__code': record.precent_item_type.code if record.precent_item_type else None,
            'precent_item_type__name': record.precent_item_type.name if record.precent_item_type else None,

            'precent_type_id': record.precent_type.id if record.precent_type else None,
            'precent_type__code': record.precent_type.code if record.precent_type else None,
            'precent_type__name': record.precent_type.name if record.precent_type else None,

            'format': record.format,
            'mime_type': record.mime_type,
            'size': record.size if not str(record.size).startswith('.') else str(record.size).replace('.', '0.'),
            'real_name': record.real_name,

            'lastmodified': record.lastmodified,
            'editing': record.editing,
            'deliting': record.deliting,
            'parent_id': record.parent.id if record.parent else None,

            'isFolder': record.isFolder,
            'enabled': IsBitOff(record.status.props, 0),
        }
        return res

    def get_queryset(self):
        return Precent_viewQuerySet(self.model, using=self._db)


class Precent_view(AuditModel):
    code = CodeField()
    isFolder = BooleanField()
    description = DescriptionField()
    parent = ForeignKeyProtect('self', null=True, blank=True, related_name='parent_p_view')

    date = DateTimeField(verbose_name='Дата', db_index=True)
    date_sign = DateTimeField(verbose_name='Дата подписания', db_index=True, null=True, blank=True)

    size = BigIntegerField(verbose_name='Размер фала', default=0)
    real_name = TextField(verbose_name='Первоначальное имя файла', null=True, blank=True, default=None)
    mime_type = NameField(verbose_name='MIME тип файла файла', null=True, blank=True, default=None)
    format = NameField(verbose_name='Формат файла')

    status = ForeignKeyProtect(StatusPrecent)
    precent_item_type = ForeignKeyProtect(Precent_item_types, null=True, blank=True)
    precent_type = ForeignKeyProtect(Precent_types, null=True, blank=True)

    objects = Precent_viewManager()

    def __str__(self):
        return f'{self.id}'

    class Meta:
        verbose_name = 'Документы'
        db_table = 'sales_precent_view'
        managed = False



# Generated by Django 3.0.3 on 2020-02-29 11:57

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0212_auto_20200225_1013'),
        ('production', '0221_auto_20200228_1240'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_item_refs',
            name='item_refs',
            field=isc_common.fields.related.ForeignKeySetNull(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='ckk.Item_refs'),
        ),
    ]



import logging

from django.core.management import BaseCommand

from isc_common import setAttr
from kaf_pas.production.models.launches import LaunchesManager

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Тестирование запуск"

    def add_arguments(self, parser):
        # parser.add_argument('--demand_id', type=int)
        parser.add_argument('--user_id', type=int)

    def handle(self, *args, **options):
        data = {'id': 6, 'code': '1', 'name': None, 'date': '2019-11-20T00:00:00.000', 'description': None, 'parent_id': None, 'demand_id': 34, 'demand__code': 'Тестовый №2', 'demand__date': '2019-10-31T00:00:00.000', 'status_id': 1, 'status__code': 'formirovanie', 'status__name': 'Формирование', 'qty': 5, 'isFolder': False, 'user_id': 2}
        # demand_id = options.get('demand_id')
        user_id = options.get('user_id')
        setAttr(data, 'user_id', user_id)

        LaunchesManager.update_launch(data=data)

        print('Done')



# Generated by Django 2.2.8 on 2019-12-18 10:13

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0181_auto_20191217_1331'),
        ('kd', '0118_auto_20191217_2231'),
    ]

    operations = [
        migrations.AddField(
            model_name='lotsman_documents_hierarcy',
            name='attr_type',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='ckk.Attr_type', verbose_name='Тип документа'),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-17 12:17

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0117_auto_20191217_0709'),
        ('ckk', '0179_item_lotsman_document'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='item',
            unique_together={('STMP_1', 'STMP_2', 'version')},
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-26 14:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0211_auto_20200226_1224'),
    ]

    operations = [
        # migrations.RemoveConstraint(
        #     model_name='resource',
        #     name='Resource_unique_constraint_0',
        # ),
        # migrations.RemoveConstraint(
        #     model_name='resource',
        #     name='Resource_unique_constraint_1',
        # ),
        migrations.AddConstraint(
            model_name='resource',
            constraint=models.UniqueConstraint(condition=models.Q(('code', None), ('name', None)), fields=('location',), name='Resource_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='resource',
            constraint=models.UniqueConstraint(condition=models.Q(code=None), fields=('location', 'name'), name='Resource_unique_constraint_1'),
        ),
        migrations.AddConstraint(
            model_name='resource',
            constraint=models.UniqueConstraint(condition=models.Q(name=None), fields=('code', 'location'), name='Resource_unique_constraint_2'),
        ),
        migrations.AddConstraint(
            model_name='resource',
            constraint=models.UniqueConstraint(fields=('code', 'location', 'name'), name='Resource_unique_constraint_3'),
        ),
    ]



# Generated by Django 2.2 on 2019-04-13 07:00

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0009_auto_20190412_2045'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item_line_refs',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='parent', to='ckk.Item_line'),
        ),
        migrations.AlterUniqueTogether(
            name='item_line_refs',
            unique_together={('child', 'parent')},
        ),
    ]



import logging

from django.db import connection, transaction
from django.db.models import PositiveIntegerField
from django.forms import model_to_dict
from isc_common.fields.related import ForeignKeyProtect
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.audit import AuditModel
from refs.models.type_param import Type_param

from kaf_pas.ckk.models.material_type import Material_type

logger = logging.getLogger(__name__)


class Material_paramsQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    def remove_view(self, **kwargs):
        material_type = Material_type.objects.get(id=kwargs.get('materials_type'))
        sql = []
        sql.append(f'DROP VIEW IF EXISTS  public.ckk_{material_type.code}_view;')

        with connection.cursor() as cursor:
            sql_str = '\n'.join(sql)
            cursor.execute(sql_str)

    def recreate_view(self, **kwargs):
        try:
            material_type = Material_type.objects.get(id=kwargs.get('materials_type'))
        except Material_type.DoesNotExist:
            material_type = Material_params.objects.get(id=kwargs.get('id')).materials_type

        sql = []
        sql.append(f'DROP VIEW IF EXISTS  public.ckk_{material_type.code}_view;')

        sql.append(f'CREATE OR REPLACE VIEW ckk_{material_type.code}_view AS ')
        sql.append('select cm.id,')

        step = 0
        for param_type in Material_params.objects.filter(materials_type=material_type).order_by('order', ):
            if step > 0:
                sql.append(",");
            step += 1
            if param_type.param_type.type == 'Строковый':
                sql.append(f"(select value_str from refs_type_param_values rv ")
            elif param_type.param_type.type == 'Целое число':
                sql.append(f"(select value_int from refs_type_param_values rv ")
            elif param_type.param_type.type == 'Дробное число':
                sql.append(f"(select value_float from refs_type_param_values rv ")
            elif param_type.param_type.type == 'Дата без времени':
                sql.append(f"(select value_date from refs_type_param_values rv ")
            elif param_type.param_type.type == 'Дата со временем':
                sql.append(f"(select value_date from refs_type_param_values rv ")
            elif param_type.param_type.type == 'Выбор из списка':
                sql.append(f"(select value_str from refs_type_param_values rv ")
            else:
                raise Exception(f'Unknown type: {param_type.param_type.type}')

            sql.append(f"join refs_type_param rtp on rtp.id = rv.type_id "
                       f"join ckk_material_added_values ca on ca.material_id = cm.id "
                       f"where rv.id = ca.param_values_id and rtp.code = '{param_type.param_type.code}')  {param_type.param_type.code}")

        sql.append("from ckk_materials cm join ckk_material_type mt on mt.id = cm.materials_type_id")
        sql.append(f"where mt.code = '{material_type.code}'")

        with connection.cursor() as cursor:
            sql_str = '\n'.join(sql)
            cursor.execute(sql_str)

    def delete(self):
        with transaction.atomic():
            for_delete = model_to_dict(super().all()[0])
            res = super().delete()
            if Material_params.objects.filter(materials_type_id=for_delete.get('materials_type')).count() > 0:
                self.recreate_view(**for_delete)
            else:
                self.remove_view(**for_delete)
            return res

    def create(self, **kwargs):
        with transaction.atomic():
            res = super().create(**kwargs)
            self.recreate_view(**model_to_dict(res))
            return res

    def update(self, **kwargs):
        with transaction.atomic():
            res = super().update(**kwargs)
            self.recreate_view(**kwargs)
            return res


class Material_paramsManager(CommonManagetWithLookUpFieldsManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'materials_type_id': record.materials_type.id,
            'materials_type__code': record.materials_type.code,
            'materials_type__name': record.materials_type.name,
            'materials_type__description': record.materials_type.description,
            'param_type_id': record.param_type.id,
            'param_type__code': record.param_type.code,
            'param_type__name': record.param_type.name,
            'param_type__type': record.param_type.type,
            'param_type__description': record.param_type.description,
            'param_type__length': record.param_type.length,
            'param_type__required': record.param_type.required,
            'order': record.order,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Material_paramsQuerySet(self.model, using=self._db)


class Material_params(AuditModel):
    materials_type = ForeignKeyProtect(Material_type)
    param_type = ForeignKeyProtect(Type_param)
    order = PositiveIntegerField(default=0)

    objects = Material_paramsManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Дополнительные параметры'
        unique_together = (('materials_type', 'param_type'),)



# Generated by Django 2.2.8 on 2019-12-12 21:52

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0175_item_refs_syntetic_parent'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='item_refs',
            unique_together={('child', 'parent')},
        ),
        migrations.RemoveField(
            model_name='item_refs',
            name='syntetic_child',
        ),
        migrations.RemoveField(
            model_name='item_refs',
            name='syntetic_parent',
        ),
    ]



from django.urls import path

from kaf_pas.planing.views import sections

urlpatterns = [

    path('Sections/Fetch/', sections.Sections_Fetch),
    path('Sections/Add', sections.Sections_Add),
    path('Sections/Update', sections.Sections_Update),
    path('Sections/Remove', sections.Sections_Remove),
    path('Sections/Lookup/', sections.Sections_Lookup),
    path('Sections/Info/', sections.Sections_Info),
    path('Sections/Copy', sections.Sections_Copy),

]



import logging

from django.db.models import Model, FloatField

from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import Manager
from isc_common.models.standard_colors import Standard_colors
from isc_common.number import DelProps
from kaf_pas.ckk.models.ed_izm import Ed_izm
from kaf_pas.ckk.models.item import Item
from kaf_pas.ckk.models.locations import Locations

logger = logging.getLogger(__name__)


class BuffersManager(Manager):
    @staticmethod
    def getValue(item):
        try:
            buffer = Buffers.objects.get(item=item)
            return buffer.value if buffer.value else 0
        except Buffers.DoesNotExist:
            return None

    @staticmethod
    def getRecord(record):
        res = {
            'location_id': record.location.id,
            'location__code': record.location.code,
            'location__name': record.location.name,
            'location__full_name': record.location.full_name,
            'item_id': record.item.id,
            'value': record.value,
            'color_id': record.color.id if record.color else None,
            'color__color': record.color.color if record.color else None,
            'color__name': record.color.name if record.color else None,
            'item__STMP_1_id': record.item.STMP_1.id if record.item.STMP_1 else None,
            'item__STMP_1__value_str': record.item.STMP_1.value_str if record.item.STMP_1 else None,
            'item__STMP_2_id': record.item.STMP_2.id if record.item.STMP_2 else None,
            'item__STMP_2__value_str': record.item.STMP_2.value_str if record.item.STMP_2 else None,
            'edizm_id': record.edizm.id,
            'edizm__code': record.edizm.code,
            'edizm__name': record.edizm.name,
        }
        return DelProps(res)


class Buffers(Model):
    item = ForeignKeyProtect(Item)
    location = ForeignKeyProtect(Locations)
    edizm = ForeignKeyProtect(Ed_izm)
    value = FloatField()
    color = ForeignKeyProtect(Standard_colors, null=True, blank=True)

    objects = BuffersManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Буферы'
        managed = False
        db_table = 'accounting_buffers_view'



import logging
import os

from bitfield import BitField
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import TextField, CharField, UniqueConstraint, Q

from isc_common import delete_drive_leter, get_drive_leter
from isc_common.bit import TurnBitOn
from isc_common.common.mat_views import refresh_mat_view
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.base_ref import Hierarcy
from isc_common.models.tree_audit import TreeAuditModelManager
from isc_common.progress import managed_progress, ProgressDroped
from kaf_pas.ckk.models.attr_type import Attr_type
from kaf_pas.production.models import progress_deleted

logger = logging.getLogger(__name__)


class PathesQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    def delete(self):
        from kaf_pas.kd.models.documents import Documents

        for item in self:
            Documents.objects.filter(path=item).delete()

        return super().delete()


class PathesManager(CommonManagetWithLookUpFieldsManager):
    @staticmethod
    def props():
        return BitField(flags=(
            ('enable_upload', 'Директория для закачки'),
        ), default=1, db_index=True)

    def get_queryset(self):
        return PathesQuerySet(self.model, using=self._db)

    @staticmethod
    def getRecord(record):
        res = {
            "id": record.id,
            "path": record.path,
            "virt_path": record.virt_path,
            "parent_id": record.parent_id,
            "lastmodified": record.lastmodified,
            "editing": record.editing,
            "drive": record.drive,
            "deliting": record.deliting,
            "attr_type_id": record.attr_type.id if record.attr_type else None,
            "attr_type__code": record.attr_type.code if record.attr_type else None,
            "attr_type__name": record.attr_type.name if record.attr_type else None,
            # "isFolder": record.isFolder,
        }
        return res

    @property
    def sep(self):
        return os.altsep if os.altsep else os.sep

    def create_ex(self, **kwargs):
        path = kwargs.get('path')
        parent = kwargs.get('parent')
        with_out_last = kwargs.get('with_out_last')

        if not path:
            raise Exception(f'path {path} is not exists.')

        if path:
            drive = get_drive_leter(path)
            path = delete_drive_leter(path)

            if path != '':
                if with_out_last:
                    path = path.split(self.sep)[: - 1]
                else:
                    path = path.split(self.sep)

                for path_item in path:
                    if path_item:
                        alive_only = self.alive_only
                        try:
                            self.alive_only = False
                            if drive and path == '':
                                parent = super().get(drive=drive, path=path_item, parent=parent)
                            else:
                                parent = super().get(path=path_item, parent=parent)
                            self.alive_only = alive_only
                        except ObjectDoesNotExist:
                            self.alive_only = alive_only
                            if drive and path == '':
                                parent = super().create(drive=drive, path=path_item, parent=parent)
                            else:
                                parent = super().create(path=path_item, parent=parent)
            else:
                parent, _ = super().get_or_create(drive=drive, parent=parent)

        return parent

    def create_ex1(self, **kwargs):
        path = kwargs.get('path')
        parent = kwargs.get('parent')
        with_out_last = kwargs.get('with_out_last')

        if not path:
            raise Exception(f'path {path} is not exists.')

        if path:
            drive = get_drive_leter(path)
            path = delete_drive_leter(path)

            if path != '':
                if with_out_last:
                    path = path.split(self.sep)
                else:
                    path = path.split(self.sep)

                for path_item in path:
                    if path_item:
                        alive_only = self.alive_only
                        try:
                            self.alive_only = False
                            if drive and path == '':
                                parent = super().get(drive=drive, path=path_item, parent=parent)
                            else:
                                parent = super().get(path=path_item, parent=parent)
                            self.alive_only = alive_only
                        except ObjectDoesNotExist:
                            self.alive_only = alive_only
                            if drive and path == '':
                                parent = super().create(drive=drive, path=path_item, parent=parent)
                            else:
                                parent = super().create(path=path_item, parent=parent)
            else:
                parent, _ = super().get_or_create(drive=drive, parent=parent)

        return parent

    # Только для перенаправленных на виндовый сервак вызовов !!!!
    def deleteFromRequest(self, request, removed=None, ):
        from django.db import transaction
        from isc_common.auth.models.user import User
        from isc_common.models.deleted_progresses import Deleted_progresses
        from kaf_pas.kd.models.documents import DocumentManager
        from kaf_pas.kd.models.documents import Documents
        from kaf_pas.kd.models.uploades import Uploades
        from kaf_pas.kd.models.uploades_documents import Uploades_documents
        from kaf_pas.kd.models.uploades_log import Uploades_log

        ids = request.GET.getlist('ids')
        request = DSRequest(request=request)
        user_id = request.user_id

        res = 0
        count = 0
        lotsman_count = 0
        try:
            for i in range(0, len(ids), 2):
                with transaction.atomic():
                    if ids[i + 1] == 'hide':
                        res += super().filter(id=ids[i]).soft_delete()
                    else:
                        cnt = Pathes.objects_tree.get_descendants_count(
                            id=ids[i],
                            child_id='id')

                        with managed_progress(
                                id=f'delete_pathes',
                                qty=cnt - 1,
                                user=request.user,
                                message='Удаление путей размещения.',
                                title='Выполнено',
                                props=TurnBitOn(0, 0)
                        ) as progress:

                            for path in Pathes.objects_tree.get_descendants(
                                    id=ids[i],
                                    child_id='id',
                                    order_by_clause='order by level desc'):

                                for upload in Uploades.objects.filter(path=path.id):
                                    Uploades_log.objects.filter(upload=upload).delete()
                                    Uploades_documents.objects.filter(upload=upload).delete()
                                    upload.delete()

                                for document in Documents.objects.filter(path_id=path.id):
                                    count, lotsman_count = DocumentManager.delete(document.id, User.objects.get(id=user_id))

                                res += super().filter(id=path.id).delete()[0]
                                logger.debug(f'Deleted: {res}')
                                step_res = progress.step()

                                if step_res != 0:
                                    raise ProgressDroped(progress_deleted)
            if count > 0:
                refresh_mat_view('kd_documents_mview')
            if lotsman_count > 0:
                refresh_mat_view('kd_lotsman_documents_hierarcy_mview')
            return res

        except ProgressDroped as ex:
            Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
            raise ex

    def get_drive(self, id):
        try:
            path = self.get(id=id)
            if path.drive:
                return path.drive
            elif path.parent:
                return self.get_drive(path.parent.id)
            else:
                return path.drive
        except Pathes.DoesNotExist:
            return None

    def get_drive(self, id):
        try:
            path = self.get(id=id)
            if path.drive:
                return path.drive
            elif path.parent:
                return self.get_drive(path.parent.id)
            else:
                return path.drive
        except Pathes.DoesNotExist:
            return None


class Pathes(Hierarcy):
    drive = CharField(verbose_name='Диск', max_length=10, null=True, blank=True)
    path = TextField(verbose_name="Путь")
    virt_path = TextField(verbose_name="Мнимый путь", null=True, blank=True)
    attr_type = ForeignKeyProtect(Attr_type, verbose_name='Атрибут', null=True, blank=True)
    props = PathesManager.props()

    @property
    def sep(self):
        return os.altsep if os.altsep else os.sep

    def get_virt_path(self, id=None):
        try:
            path = Pathes.objects.get(id=id if id else self.id)
            if path.virt_path:
                return path.virt_path
            elif path.parent:
                return self.get_virt_path(path.parent.id)
            else:
                return path.virt_path
        except Pathes.DoesNotExist:
            return None

    @property
    def absolute_path(self):
        def get_parent(item_tuple):
            if item_tuple[0].parent:
                res = Pathes.objects.get(id=item_tuple[0].parent.id)
                res = (res, f"{res.path}/{item_tuple[1]}")
                return get_parent(res)
            else:
                return item_tuple

        if self.parent:
            res = get_parent((self, self.path))
            return f'{self.sep}{res[1]}'
        else:
            return f'{self.sep}{self.path}'

    def get_drive(self, id=None):
        try:
            path = Pathes.objects.get(id=id if id else self.id)
            if path.drive:
                return path.drive
            elif path.parent:
                return self.get_drive(path.parent.id)
            else:
                return path.drive
        except Pathes.DoesNotExist:
            return None

    @property
    def drived_absolute_path(self):
        drive = self.get_drive(self.id)
        if drive:
            return f'{drive}{self.absolute_path}'
        else:
            return self.absolute_path

    def __str__(self):
        return f"{self.absolute_path}"

    objects = PathesManager()
    objects_tree = TreeAuditModelManager()

    class Meta:
        db_table = 'pathes'
        verbose_name = 'Пути нахождения документов'
        constraints = [
            UniqueConstraint(fields=['id'], condition=Q(parent=None), name='Pathes_unique_constraint_0'),
            UniqueConstraint(fields=['id', 'parent'], name='Pathes_unique_constraint_1'),
        ]



from django.urls import path

from kaf_pas.input.views import user_view

urlpatterns = [

    path('User_view/Fetch/', user_view.User_view_Fetch),
    path('User_view/Add', user_view.User_view_Add),
    path('User_view/Update', user_view.User_view_Update),
    path('User_view/Remove', user_view.User_view_Remove),
    path('User_view/Lookup/', user_view.User_view_Lookup),
    path('User_view/Info/', user_view.User_view_Info),

]



import logging

from django.db import transaction
from django.db.models import DateTimeField
from django.forms import model_to_dict
from django.utils import timezone

from crypto.models.crypto_file import Crypto_file, CryptoManager, CryptoQuerySet
from isc_common import delAttr, setAttr
from isc_common.fields.code_field import CodeField
from isc_common.fields.description_field import DescriptionField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from kaf_pas.sales.models.precent_item_types import Precent_item_types
from kaf_pas.sales.models.precent_types import Precent_types
from kaf_pas.sales.models.status_precent import StatusPrecent

logger = logging.getLogger(__name__)


class PrecentQuerySet(CryptoQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class PrecentManager(CryptoManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'date': record.date,
            'date_sign': record.date_sign,
            'description': record.description,

            'status_id': record.status.id,
            'status__code': record.status.code,
            'status__name': record.status.name,

            'format': record.format,
            'mime_type': record.mime_type,
            'size': record.size if not str(record.size).startswith('.') else str(record.size).replace('.', '0.'),
            'real_name': record.real_name,

            'lastmodified': record.lastmodified,
            'editing': record.editing,
            'deliting': record.deliting,
            'parent_id': record.parent.id if record.parent else None,

            'precent_item_type_id': record.precent_item_type.id,
            'precent_item_type__code': record.precent_item_type.code,
            'precent_item_type__name': record.precent_item_type.name,

            'precent_type_id': record.precent_type.id,
            'precent_type__code': record.precent_type.code,
            'precent_type__name': record.precent_type.name,
        }
        return res

    def createFromRequest(self, request, printRequest=False, function=None):
        from kaf_pas.sales.models.precent_dogovors import Precent_dogovors
        from kaf_pas.sales.models.status_dogovor import StatusDogovor
        from kaf_pas.sales.models.dogovor_types import Dogovor_types
        from kaf_pas.sales.models.customer import Customer
        from kaf_pas.sales.models.dogovors import Dogovors

        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        for key in data:
            if key.find('__') != -1:
                delAttr(_data, key)
        delAttr(_data, 'form')

        with transaction.atomic():
            document = data.get('document')
            if not isinstance(document, list):
                status, _ = StatusDogovor.objects.get_or_create(code='virtual', props=0)
                customer, _ = Customer.objects.get_or_create(name='virtual', props=0)
                dogovor_type, _ = Dogovor_types.objects.get_or_create(code='virtual', props=0)
                dogovor, _ = Dogovors.objects.get_or_create(name='virtual', status=status, customer=customer, type=dogovor_type, props=0, deliting=False, editing=False)
                document = [dogovor.id]

            else:
                delAttr(_data, 'document')
            for document_id in document:
                if data.get('id') or not data.get('real_name'):
                    delAttr(_data, 'id')
                    setAttr(_data, 'precent_item_type', Precent_types.objects.get(id=_data.get( 'precent_type_id')).precent_item_type)
                    res, _ = super().get_or_create(id=data.get('id'), defaults=_data)

                    Precent_dogovors.objects.get_or_create(dogovor_id=document_id, precent=res)
                    res = model_to_dict(res)
                    delAttr(res, 'attfile')
                    delAttr(res, 'form')
                    data.update(res)

        return data

    def updateFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        # data.update(dict(type))
        delAttr(_data, 'status__code')
        delAttr(_data, 'status__name')
        delAttr(_data, 'precent_item_type__code')
        delAttr(_data, 'precent_item_type__name')
        delAttr(_data, 'type__name')
        delAttr(_data, 'ifFolder')
        delAttr(_data, 'form')
        super().filter(id=request.get_id()).update(**_data)
        return data

    def get_queryset(self):
        return PrecentQuerySet(self.model, using=self._db)


class Precent(Crypto_file):
    code = CodeField()
    description = DescriptionField()
    parent = ForeignKeyProtect('self', null=True, blank=True, related_name='parent_p')

    date = DateTimeField(verbose_name='Дата', db_index=True, default=timezone.now)
    date_sign = DateTimeField(verbose_name='Дата подписания', db_index=True, null=True, blank=True)

    status = ForeignKeyProtect(StatusPrecent)
    precent_item_type = ForeignKeyProtect(Precent_item_types)
    precent_type = ForeignKeyProtect(Precent_types)

    objects = PrecentManager()

    def __str__(self):
        return f'{self.id}'

    class Meta:
        verbose_name = 'Распоряжение'



from django.urls import path

from kaf_pas.kd.views import document_attr_cross

urlpatterns = [

    path('Document_attr_cross/Fetch/', document_attr_cross.Document_attr_cross_Fetch),
    path('Document_attr_cross/Add', document_attr_cross.Document_attr_cross_Add),
    path('Document_attr_cross/Update', document_attr_cross.Document_attr_cross_Update),
    path('Document_attr_cross/Remove', document_attr_cross.Document_attr_cross_Remove),
    path('Document_attr_cross/Lookup/', document_attr_cross.Document_attr_cross_Lookup),
    path('Document_attr_cross/Info/', document_attr_cross.Document_attr_cross_Info),

]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.kd.models.spw_attrs import Spw_attrs, Spw_attrsManager


@JsonResponseWithException()
def Spw_attrs_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Spw_attrs.objects.
                select_related('attr_type').
                get_range_rows1(
                request=request,
                function=Spw_attrsManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)

@JsonResponseWithException()
def Spw_attrs_Update(request):
    return JsonResponse(DSResponseUpdate(data=Spw_attrs.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


# @JsonResponseWithException()
# def Spw_attrs_Remove(request):
#     return JsonResponse(DSResponse(request=request, data=Spw_attrs.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Spw_attrs_Info(request):
    return JsonResponse(DSResponse(request=request, data=Spw_attrs.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)




# Generated by Django 2.1.7 on 2019-03-27 07:03

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('kd', '0001_initial'),
        ('ckk', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='item_line',
            name='SPC_CLM_COUNT',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_COUNT', to='kd.Document_attrs', verbose_name='Количество'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='SPC_CLM_FACTORY',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_FACTORY', to='kd.Document_attrs', verbose_name='Предприятие - изготовитель'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='SPC_CLM_FORMAT',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_FORMAT', to='kd.Document_attrs', verbose_name='Формат'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='SPC_CLM_KOD',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_KOD', to='kd.Document_attrs', verbose_name='Код'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='SPC_CLM_MARK',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_MARK', to='kd.Document_attrs', verbose_name='Обозначение'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='SPC_CLM_MASSA',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_MASSA', to='kd.Document_attrs', verbose_name='Масса'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='SPC_CLM_MATERIAL',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_MATERIAL', to='kd.Document_attrs', verbose_name='Материал'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='SPC_CLM_NAME',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_NAME', to='kd.Document_attrs', verbose_name='Наименование'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='SPC_CLM_NOTE',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_NOTE', to='kd.Document_attrs', verbose_name='Примечание'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='SPC_CLM_POS',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_POS', to='kd.Document_attrs', verbose_name='Позиция'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='SPC_CLM_USER',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_USER', to='kd.Document_attrs', verbose_name='Пользовательская'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='SPC_CLM_ZONE',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_ZONE', to='kd.Document_attrs', verbose_name='Зона'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='document',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='kd.Documents', verbose_name='Документ'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='item_child',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='item_child', to='ckk.Item'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='item_parent',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='item_parent', to='ckk.Item'),
        ),
        migrations.AddField(
            model_name='item',
            name='document',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='kd.Documents', verbose_name='Документ'),
        ),
        migrations.AddField(
            model_name='attr_type',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Attr_type'),
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-21 09:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0014_auto_20191121_0623'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_item_refs_timing',
            name='time_duration_operations',
            field=models.DecimalField(blank=True, decimal_places=4, default=0.0, max_digits=19, null=True),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-08 19:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0037_document_attrs_view'),
    ]

    operations = [
        migrations.AlterField(
            model_name='documents',
            name='file_access_time',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата время поcледнего доступа к документу'),
        ),
        migrations.AlterField(
            model_name='documents',
            name='file_change_time',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата время изменнения документа'),
        ),
        migrations.AlterField(
            model_name='documents',
            name='file_document',
            field=models.TextField(db_index=True, verbose_name='Полный путь к файлу'),
        ),
        migrations.AlterField(
            model_name='documents',
            name='file_modification_time',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата время поcледнего модификации документа'),
        ),
    ]



import logging

import requests
from django.conf import settings
from django.http import HttpResponseRedirect

from isc_common.http.DSRequest import DSRequest
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from isc_common.logger.Logger import Logger
from kaf_pas.kd.models.documents import Documents, DocumentManager

logger = logging.getLogger(__name__)
logger.__class__ = Logger


@JsonResponseWithException()
def Documents_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Documents.objects.
                select_related('attr_type').
                get_range_rows1(
                request=request,
                function=DocumentManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_Add(request):
    return JsonResponse(DSResponseAdd(data=Documents.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_Update(request):
    return JsonResponse(DSResponseUpdate(data=Documents.objects.updateFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_Remove(request):
    ws_channel = request.session.get('ws_channel')
    ws_port = settings.WS_PORT
    host = settings.WS_HOST

    _request = DSRequest(request=request)
    ids = _request.get_tuple_ids()
    r = requests.get(f'{settings.KOMPAS_INFORMICA}/logic/Documents/Remove', params=dict(ids=ids, port=ws_port, ws_channel=ws_channel, host=host))
    HttpResponseRedirect(r.url)
    return JsonResponse(DSResponse(request=request, status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Documents.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_Info(request):
    return JsonResponse(DSResponse(request=request, data=Documents.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_MakeItem(request):
    from kaf_pas.ckk.models.attr_type import Attr_type
    from kaf_pas.kd.models.documents_ext import DocumentManagerExt

    _request = DSRequest(request=request)
    ids = _request.get_data()
    if isinstance(ids, dict):
        for id in ids.get('ids'):
            document = Documents.objects.get(id=id)
            if document.attr_type.code == 'SPW':

                DocumentManagerExt(logger=logger).make_spw(document=document)

            elif document.attr_type.code == 'CDW':
                DocumentManagerExt(logger=logger).make_cdw(document=document)

            elif document.attr_type.code == 'KD_PDF':

                STMP_1_type = Attr_type.objects.get(code='STMP_1')
                STMP_2_type = Attr_type.objects.get(code='STMP_2')

                DocumentManagerExt(logger=logger).make_pdf(document=document, STMP_1_type=STMP_1_type, STMP_2_type=STMP_2_type)
    return JsonResponse(DSResponse(request=request, status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_Treat(request):
    ws_channel = request.session.get('ws_channel')
    ws_port = settings.WS_PORT
    host = settings.WS_HOST

    _request = DSRequest(request=request)
    ids = _request.get_tuple_ids()
    r = requests.get(f'{settings.KOMPAS_INFORMICA}/logic/Documents/Treat', params=dict(ids=ids, port=ws_port, ws_channel=ws_channel, host=host, user_id=_request.user_id))
    HttpResponseRedirect(r.url)
    return JsonResponse(DSResponse(request=request, status=RPCResponseConstant.statusSuccess).response) @ JsonResponseWithException()


@JsonResponseWithException()
def Documents_ReloadDoc(request):
    ws_channel = request.session.get('ws_channel')
    ws_port = settings.WS_PORT
    host = settings.WS_HOST

    _request = DSRequest(request=request)
    ids = _request.json.get('data')
    ids = ids.get('ids')

    r = requests.get(f'{settings.KOMPAS_INFORMICA}/logic/Documents/ReloadDoc', params=dict(ids=ids, port=ws_port, ws_channel=ws_channel, host=host, user_id=_request.user_id))
    HttpResponseRedirect(r.url)
    return JsonResponse(DSResponse(request=request, status=RPCResponseConstant.statusSuccess).response)



import logging
import os
from os.path import getsize

from bitfield import BitField
from django.conf import settings
from django.db import transaction
from django.db.models import DateTimeField, TextField, PositiveIntegerField

from isc_common import delAttr, replace_alt_set
from isc_common.common.mat_views import refresh_mat_view, create_tmp_mat_view, drop_mat_view
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.audit import AuditModel
from isc_common.progress import Progress
from kaf_pas.ckk.models.attr_type import Attr_type
from kaf_pas.kd.models.pathes import Pathes

logger = logging.getLogger(__name__)


class DocumentQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    pass


class DocumentManager(CommonManagetWithLookUpFieldsManager):
    type_bad = Attr_type.objects.get_attr(code='KD_BAD')

    def get_queryset(self):
        return DocumentQuerySet(self.model, using=self._db)

    @staticmethod
    def get_m_view():
        sql_str = '''SELECT kd.id,
                            kd.editing,
                            kd.deliting,
                            kd.lastmodified,
                            kd.file_document,
                            kd.file_size,
                            kd.file_modification_time,
                            kd.file_access_time,
                            kd.file_change_time,
                            kd.attr_type_id,
                            kd.path_id,
                            kd.props,
                            kd.deleted_at,
                            kd.load_error,
                            ( SELECT kat.id
                                   FROM kd_document_attributes kat
                                     JOIN kd_document_attr_cross dc ON kat.id = dc.attribute_id
                                     RIGHT JOIN ckk_attr_type att ON att.id = kat.attr_type_id
                                  WHERE dc.document_id = kd.id AND att.code::text = 'STMP_1'::text) AS "STMP_1_id",
                            ( SELECT kat.id
                                   FROM kd_document_attributes kat
                                     JOIN kd_document_attr_cross dc ON kat.id = dc.attribute_id
                                     RIGHT JOIN ckk_attr_type att ON att.id = kat.attr_type_id
                                  WHERE dc.document_id = kd.id AND att.code::text = 'STMP_2'::text) AS "STMP_2_id",
                            relevant_column(kd.props::integer) AS relevant
                           FROM kd_documents kd'''

        drop_mat_view(mat_view_name='kd_documents_mview')
        create_tmp_mat_view(sql_str=sql_str, mat_view_name='kd_documents_mview', indexes=['attr_type_id', 'path_id', 'STMP_1_id', 'STMP_2_id'])

    @staticmethod
    def getRecord(record):
        res = {
            "id": record.id,
            "path_id": record.path.id,
            "attr_type_id": record.attr_type.id,
            "attr_type__code": record.attr_type.code,
            "attr_type__name": record.attr_type.name,
            "file_document": record.file_document,
            "file_size": record.file_size,
            "file_modification_time": record.file_modification_time,
            "file_access_time": record.file_access_time,
            "file_change_time": record.file_change_time,
            "lastmodified": record.lastmodified,
            'load_error': f'<pre>{record.relevant}</pre>' if record.relevant else None,
            "editing": record.editing,
            "deliting": record.deliting,
        }
        return res

    def get_ex(self, *args, **kwargs):
        file_name = kwargs.get('file_document')
        file_drive = kwargs.get('file_drive')
        virt_path = kwargs.get('virt_path')
        attr_type = kwargs.get('attr_type')
        delAttr(kwargs, 'file_document')

        if virt_path:
            if virt_path[0:1] == os.sep or virt_path[0:1] == os.altsep:
                virt_path = virt_path[1:]

        if virt_path:
            if file_name.find(virt_path) == -1:
                _file_name = f'{virt_path}{os.sep}{file_name}'.replace(os.altsep, os.sep)
            else:
                _file_name = file_name.replace(os.altsep, os.sep)
        else:
            _file_name = file_name.replace(os.altsep, os.sep)

        for doc in Documents.objects.filter(path=kwargs.get('path'), attr_type=attr_type, props=Documents.props.relevant).exclude(attr_type=self.type_bad):
            real_file = replace_alt_set(f'{file_drive}{os.sep}{file_name}')

            if replace_alt_set(doc.file_document) == replace_alt_set(_file_name):
                real_file_size = getsize(real_file.replace(replace_alt_set(virt_path), '') if virt_path else real_file)

                # real_file_modification_time = datetime.fromtimestamp(getmtime(real_file)).replace(microsecond=0)
                # system_file_modification_time = doc.file_modification_time.replace(microsecond=0)

                # real_file_access_time = datetime.fromtimestamp(getatime(real_file)).replace(microsecond=0)
                # system_file_access_time = doc.file_access_time.replace(microsecond=0)
                #
                # real_file_change_time = datetime.fromtimestamp(getctime(real_file)).replace(microsecond=0)
                # system_file_change_time = doc.file_change_time.replace(microsecond=0)

                if doc.file_size != real_file_size:
                    # logger.debug(f'Неравенство размеров {doc.file_size} != {real_file_size}')
                    return doc, True
                # elif system_file_modification_time != real_file_modification_time:
                #     logger.debug(f'Неравенство file_modification_time {system_file_modification_time} != {real_file_modification_time}')
                #     return doc, True
                # elif system_file_access_time != real_file_access_time:
                #     logger.debug(f'Неравенство file_access_time {system_file_access_time} != {real_file_access_time}')
                #     return doc, True
                # elif system_file_change_time != real_file_change_time:
                #     logger.debug(f'Неравенство file_change_time {system_file_change_time} != {real_file_change_time}')
                #     return doc, True
                else:
                    return doc, False
        return None, None

    def get_ex1(self, *args, **kwargs):
        file_name = kwargs.get('file_document')
        file_drive = kwargs.get('file_drive')
        virt_path = kwargs.get('virt_path')
        attr_type = kwargs.get('attr_type')
        delAttr(kwargs, 'file_document')

        if virt_path:
            if virt_path[0:1] == os.sep or virt_path[0:1] == os.altsep:
                virt_path = virt_path[1:]

        if virt_path:
            if file_name.find(virt_path) == -1:
                _file_name = f'{virt_path}{os.sep}{file_name}'.replace(os.altsep, os.sep)
            else:
                _file_name = file_name.replace(os.altsep, os.sep)
        else:
            _file_name = file_name.replace(os.altsep, os.sep)

        for doc in Documents.objects.filter(path=kwargs.get('path'), attr_type=attr_type).exclude(attr_type=self.type_bad):
            real_file = replace_alt_set(f'{file_drive}{os.sep}{file_name}')

            if replace_alt_set(doc.file_document) == replace_alt_set(_file_name):
                real_file_size = getsize(real_file.replace(replace_alt_set(virt_path), '') if virt_path else real_file)

                # real_file_modification_time = datetime.fromtimestamp(getmtime(real_file)).replace(microsecond=0)
                # system_file_modification_time = doc.file_modification_time.replace(microsecond=0)

                # real_file_access_time = datetime.fromtimestamp(getatime(real_file)).replace(microsecond=0)
                # system_file_access_time = doc.file_access_time.replace(microsecond=0)
                #
                # real_file_change_time = datetime.fromtimestamp(getctime(real_file)).replace(microsecond=0)
                # system_file_change_time = doc.file_change_time.replace(microsecond=0)

                if doc.file_size != real_file_size:
                    # logger.debug(f'Неравенство размеров {doc.file_size} != {real_file_size}')
                    return doc, True
                # elif system_file_modification_time != real_file_modification_time:
                #     logger.debug(f'Неравенство file_modification_time {system_file_modification_time} != {real_file_modification_time}')
                #     return doc, True
                # elif system_file_access_time != real_file_access_time:
                #     logger.debug(f'Неравенство file_access_time {system_file_access_time} != {real_file_access_time}')
                #     return doc, True
                # elif system_file_change_time != real_file_change_time:
                #     logger.debug(f'Неравенство file_change_time {system_file_change_time} != {real_file_change_time}')
                #     return doc, True
                else:
                    # Для импорта из Лоцмана т.к. состоит из многих узлов,  которые закачиваются сами по себе
                    return doc, False
        return None, None

    def updateFromRequest(self, request, removed=None, function=None):
        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()

        relevant = data.get('relevant')
        props = data.get('props')

        if relevant == 'Актуален':
            props |= Documents.props.relevant
        else:
            props &= ~Documents.props.relevant

        res = super().get(id=data.get('id'))
        res.props = props
        res.save()
        refresh_mat_view('kd_documents_mview')

        return data

    @staticmethod
    def delete(id, user):
        from isc_common.auth.models.user import User
        from kaf_pas.ckk.models.item import Item
        from kaf_pas.ckk.models.item import ItemManager
        from kaf_pas.kd.models.document_attr_cross import Document_attr_cross
        from kaf_pas.kd.models.documents_thumb import Documents_thumb
        from kaf_pas.kd.models.documents_thumb10 import Documents_thumb10
        from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy
        from kaf_pas.kd.models.lotsman_documents_hierarcy_files import Lotsman_documents_hierarcy_files
        from kaf_pas.kd.models.lotsman_documents_hierarcy_refs import Lotsman_documents_hierarcy_refs
        from kaf_pas.kd.models.uploades_documents import Uploades_documents
        from kaf_pas.kd.models.documents_history import Documents_history

        if not isinstance(user, User):
            raise Exception(f'user  must be a User instance.')

        settings.LOCKS.acquire('DocumentManager.delete')
        res = 0
        lotsman_res = 0

        for item in Item.objects.filter(document_id=id):
            if ItemManager.delete_recursive(item_id=item.id, user=user, delete_lines=True, document=Documents.objects.get(id=id)) == -2:
                return res, lotsman_res

        for lotsman_documents_hierarcy in Lotsman_documents_hierarcy.objects.filter(document_id=id):
            for item in Item.objects.filter(lotsman_documents_hierarcy=lotsman_documents_hierarcy):
                if ItemManager.delete_recursive(item_id=item.id, user=user, delete_lines=True, document=lotsman_documents_hierarcy) == 2:
                    return res, lotsman_res

            Lotsman_documents_hierarcy_refs.objects.filter(child=lotsman_documents_hierarcy).delete()
            Lotsman_documents_hierarcy_refs.objects.filter(parent=lotsman_documents_hierarcy).delete()
            lotsman_documents_hierarcy.delete()
            lotsman_res += 1

        Documents_history.objects.filter(new_document_id=id).delete()
        Documents_history.objects.filter(old_document_id=id).delete()
        Uploades_documents.objects.filter(document_id=id).delete()
        Documents_thumb.objects.filter(document_id=id).delete()
        Documents_thumb10.objects.filter(document_id=id).delete()
        Document_attr_cross.objects.filter(document_id=id).delete()

        Lotsman_documents_hierarcy_files.objects.filter(lotsman_document__document_id=id).delete()

        Documents_thumb.objects.filter(lotsman_document__document_id=id).delete()
        Documents_thumb10.objects.filter(lotsman_document__document_id=id).delete()

        res += Documents.objects.filter(id=id).delete()[0]
        settings.LOCKS.release('DocumentManager.delete')
        return res, lotsman_res

    def deleteFromRequest(self, request):
        from kaf_pas.ckk.models.item import Item
        from kaf_pas.ckk.models.item_image_refs import Item_image_refs
        from kaf_pas.kd.models.documents_thumb import Documents_thumb
        from kaf_pas.kd.models.documents_thumb10 import Documents_thumb10
        from isc_common.auth.models.user import User

        ids = request.GET.getlist('ids')
        request = DSRequest(request=request)
        user_id = request.user_id

        res = 0
        lotsman_res = 0
        with transaction.atomic():
            for i in range(0, len(ids), 2):
                id = ids[i]
                visibleMode = ids[i + 1]

                if visibleMode != "none":
                    for item in Item.objects.filter(document_id=id):
                        Item_image_refs.objects.filter(item=item).soft_delete(visibleMode=visibleMode)

                        Documents_thumb.objects.filter(lotsman_document__document_id=id).soft_delete(visibleMode=visibleMode)
                        Documents_thumb10.objects.filter(lotsman_document__document_id=id).soft_delete(visibleMode=visibleMode)

                        Documents_thumb.objects.filter(document_id=id).soft_delete(visibleMode=visibleMode)
                        Documents_thumb10.objects.filter(document_id=id).soft_delete(visibleMode=visibleMode)

                    Item.objects.filter(document_id=id).soft_delete(visibleMode=visibleMode)
                    super().filter(id=id).soft_delete(visibleMode=visibleMode)
                    res += super().filter(id=id).soft_delete(visibleMode=visibleMode)
                else:
                    _res, _lotsman_res = DocumentManager.delete(id, User.objects.get(id=user_id))
                    res += _res
                    lotsman_res += _lotsman_res

        if res > 0:
            refresh_mat_view(mat_view_name='kd_documents_mview', progressStack=Progress(user=user_id, id='kd_documents_mview'))
        if lotsman_res > 0:
            refresh_mat_view(mat_view_name='kd_lotsman_documents_hierarcy_mview', progressStack=Progress(user=user_id, id='kd_lotsman_documents_hierarcy_mview'))
        return res


class Documents(AuditModel):
    path = ForeignKeyProtect(Pathes, verbose_name='Путь к документу')
    attr_type = ForeignKeyProtect(Attr_type, verbose_name='Тип документа')
    file_document = TextField(verbose_name='Полный путь к файлу', db_index=True)
    file_size = PositiveIntegerField(verbose_name='Размер файла')
    file_modification_time = DateTimeField(verbose_name='Дата время поcледнего модификации документа', null=True, blank=True, db_index=True)
    file_access_time = DateTimeField(verbose_name='Дата время поcледнего доступа к документу', null=True, blank=True, db_index=True)
    file_change_time = DateTimeField(verbose_name='Дата время изменнения документа', null=True, blank=True, db_index=True)
    props = BitField(flags=(
        ('relevant', 'Актуальность'),
        ('beenItemed', 'Был внесен в состав изделий'),
    ), default=1, db_index=True)
    load_error = TextField(null=True, blank=True)

    @property
    def file_name(self):
        (_, filename) = os.path.split(self.file_document)
        return filename

    objects = DocumentManager()

    @property
    def is_cdw(self):
        return self.attr_type.code == 'CDW'

    @property
    def is_spw(self):
        return self.attr_type.code == 'CPW'

    @property
    def is_pdf(self):
        dir, file = os.path.split(self.file_document)
        _, ext = os.path.splitext(file)
        if ext.lower() == '.pdf':
            return True
        else:
            return False

    def __str__(self):
        return f"ID: {self.id}, " \
               f"{self.file_document}, " \
               f"attr_type: [{self.attr_type}], " \
               f"file_document: {self.file_document}, " \
               f"file_size: {self.file_size}, " \
               f"file_modification_time: {self.file_modification_time}, " \
               f"file_access_time: {self.file_access_time}, " \
               f"file_change_time: {self.file_change_time}, " \
               f"props: {self.props}, " \
               f"load_error: {self.load_error}"

    class Meta:
        verbose_name = 'Конструкторские документы'



from django.urls import path

from kaf_pas.kd.views import document_attributes_STMP_2

urlpatterns = [

    path('Document_attributes_STMP_2/Fetch/', document_attributes_STMP_2.Document_attributes_STMP_2_Fetch),
    path('Document_attributes_STMP_2/Add', document_attributes_STMP_2.Document_attributes_STMP_2_Add),
    path('Document_attributes_STMP_2/Update', document_attributes_STMP_2.Document_attributes_STMP_2_Update),
    path('Document_attributes_STMP_2/Remove', document_attributes_STMP_2.Document_attributes_STMP_2_Remove),
    path('Document_attributes_STMP_2/Lookup/', document_attributes_STMP_2.Document_attributes_STMP_2_Lookup),
    path('Document_attributes_STMP_2/Info/', document_attributes_STMP_2.Document_attributes_STMP_2_Info),
    path('Document_attributes_STMP_2/Copy', document_attributes_STMP_2.Document_attributes_STMP_2_Copy),

]



from django.urls import path

from kaf_pas.sales.views import status_demand

urlpatterns = [

    path('Status_demand/Fetch/', status_demand.Status_demand_Fetch),
    path('Status_demand/Add', status_demand.Status_demand_Add),
    path('Status_demand/Update', status_demand.Status_demand_Update),
    path('Status_demand/Remove', status_demand.Status_demand_Remove),
    path('Status_demand/Lookup/', status_demand.Status_demand_Lookup),
    path('Status_demand/Info/', status_demand.Status_demand_Info),
    path('Status_demand/Copy', status_demand.Status_demand_Copy),

]



import logging

from django.db.models import Model

from isc_common.common.mat_views import create_tmp_table
from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditQuerySet, AuditManager
from isc_common.models.tree_audit import TreeAuditModelManager
from isc_common.number import DelProps
from kaf_pas.ckk.models.item import Item
from kaf_pas.ckk.models.item_refs import Item_refsManager

logger = logging.getLogger(__name__)


class Tmp_Item_refsQuerySet(AuditQuerySet):
    pass


class Tmp_Item_refsManager(AuditManager):

    @staticmethod
    def create():
        create_tmp_table(
            fields_str='''id bigserial NOT NULL,
                        child_id int8 NOT NULL,
                        parent_id int8 NULL,
                        props int8 NOT NULL,
                        CONSTRAINT "Tmp_Item_refs_not_circulate_refs" CHECK ((NOT (child_id = parent_id))),
                        CONSTRAINT "Tmp_Item_refs_unique" UNIQUE (child_id, parent_id),
                        CONSTRAINT ckk_tmp_item_refs_pkey PRIMARY KEY (id)''',
            unique_indexes=[('child_id', 'parent_id IS NULL')],
            table_name='ckk_tmp_item_refs',
            # on_commit=None,
            drop=False
        )

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'child_id': record.child.id,
            'parent_id': record.parent.id if record.parent else None,
            'used': record.props.used,
        }
        return DelProps(res)

    def get_queryset(self):
        return Tmp_Item_refsQuerySet(self.model, using=self._db)


class Tmp_Item_refs(Model):
    child = ForeignKeyProtect(Item, related_name='tmp_child')
    parent = ForeignKeyProtect(Item, related_name='tmp_parent', blank=True, null=True)

    props = Item_refsManager.props()

    objects = TreeAuditModelManager()

    def __str__(self):
        return f'\nID={self.id}, child=[{self.child}], parent=[{self.parent}]'

    class Meta:
        managed = False
        db_table = 'ckk_tmp_item_refs'
        verbose_name = 'Tmp_Item_refs'



from django.urls import path

from kaf_pas.kd.views import document_attributes

urlpatterns = [
    path('Document_attributes/Fetch/', document_attributes.Document_attributes_Fetch),
    path('Document_attributes/Add', document_attributes.Document_attributes_Add),
    path('Document_attributes/Update', document_attributes.Document_attributes_Update),
    path('Document_attributes/Remove', document_attributes.Document_attributes_Remove),
    path('Document_attributes/Lookup/', document_attributes.Document_attributes_Lookup),
    path('Document_attributes/Info/', document_attributes.Document_attributes_Info),
]



# Generated by Django 3.0.3 on 2020-02-08 07:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0193_auto_20200208_0714'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='item',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='item',
            constraint=models.UniqueConstraint(condition=models.Q(STMP_1=None), fields=('STMP_1', 'STMP_2', 'version', 'lotsman_type_id'), name='unique_with_optional'),
        ),
    ]



import logging

from django.db.models import PositiveIntegerField

import kaf_pas
from isc_common.fields.related import ForeignKeyCascade, ForeignKeyProtect
from isc_common.models.audit import AuditManager, AuditQuerySet, AuditModel
from kaf_pas import planing, production
from kaf_pas.ckk.models.ed_izm import Ed_izm
from kaf_pas.planing.models.operations import Operations
from kaf_pas.production.models.operations import Operations

logger = logging.getLogger(__name__)


class Operation_operationQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Operation_operationManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Operation_operationQuerySet(self.model, using=self._db)


class Operation_operation(AuditModel):
    operation = ForeignKeyCascade(kaf_pas.planing.models.operations.Operations, related_name='planing_operation_2')
    production_operation = ForeignKeyCascade(kaf_pas.production.models.operations.Operations, related_name='production_operation_2')
    num = PositiveIntegerField()
    qty = PositiveIntegerField(null=True, blank=True)
    ed_izm = ForeignKeyProtect(Ed_izm, null=True, blank=True)

    objects = Operation_operationManager()

    def __str__(self):
        return f'ID:{self.id}, operation: [{self.operation}], production_operation: [{self.production_operation}]'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Кросс-таблица'



# Generated by Django 2.2.7 on 2019-11-17 04:48

import django.utils.timezone
from django.db import migrations, models

import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0085_auto_20191031_1745'),
        ('ckk', '0158_delete_status'),
        ('sales', '0112_auto_20191117_0448'),
        ('production', '0086_operations_item_description'),
    ]

    operations = [
        migrations.CreateModel(
            name='Launches',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('code', isc_common.fields.code_field.CodeField(blank=True, null=True)),
                ('name', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('description', isc_common.fields.description_field.DescriptionField()),
                ('date', models.DateTimeField()),
                ('qty', models.PositiveIntegerField()),
                ('demand', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='sales.Demand')),
                ('parent', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='production.Launches')),
                ('status', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='sales.Status_launch')),
            ],
            options={
                'verbose_name': 'Запуски',
            },
        ),
        migrations.CreateModel(
            name='Launch_items',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('item', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Item')),
                ('launch', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='production.Launches')),
            ],
            options={
                'verbose_name': 'Товарные позиции в запуске',
            },
        ),
        migrations.CreateModel(
            name='Launch_item_refs',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('child', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='launch_child', to='production.Launch_items')),
                ('item_refs', isc_common.fields.related.ForeignKeySetNull(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='ckk.Item_refs')),
                ('parent', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='launch_parent', to='production.Launch_items')),
            ],
            options={
                'verbose_name': 'Кросс таблица',
            },
        ),
        migrations.CreateModel(
            name='Launch_item_line',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('section', isc_common.fields.code_field.CodeStrictField()),
                ('section_num', models.PositiveIntegerField(blank=True, null=True)),
                ('subsection', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('SPC_CLM_COUNT', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Launch_SPC_CLM_COUNT', to='kd.Document_attributes', verbose_name='Количество')),
                ('SPC_CLM_FACTORY', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Launch_SPC_CLM_FACTORY', to='kd.Document_attributes', verbose_name='Предприятие - изготовитель')),
                ('SPC_CLM_FORMAT', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Launch_SPC_CLM_FORMAT', to='kd.Document_attributes', verbose_name='Формат')),
                ('SPC_CLM_KOD', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Launch_SPC_CLM_KOD', to='kd.Document_attributes', verbose_name='Код')),
                ('SPC_CLM_MARK', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Launch_SPC_CLM_MARK', to='kd.Document_attributes', verbose_name='Обозначение')),
                ('SPC_CLM_MASSA', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Launch_SPC_CLM_MASSA', to='kd.Document_attributes', verbose_name='Масса')),
                ('SPC_CLM_MATERIAL', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Launch_SPC_CLM_MATERIAL', to='kd.Document_attributes', verbose_name='Материал')),
                ('SPC_CLM_NAME', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Launch_SPC_CLM_NAME', to='kd.Document_attributes', verbose_name='Наименование')),
                ('SPC_CLM_NOTE', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Launch_SPC_CLM_NOTE', to='kd.Document_attributes', verbose_name='Примечание')),
                ('SPC_CLM_POS', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Launch_SPC_CLM_POS', to='kd.Document_attributes', verbose_name='Позиция')),
                ('SPC_CLM_USER', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Launch_SPC_CLM_USER', to='kd.Document_attributes', verbose_name='Пользовательская')),
                ('SPC_CLM_ZONE', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Launch_SPC_CLM_ZONE', to='kd.Document_attributes', verbose_name='Зона')),
                ('child', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='Launch_item_child', to='production.Launch_items', verbose_name='Товарная позиция')),
                ('item_line', isc_common.fields.related.ForeignKeySetNull(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='ckk.Item_line')),
                ('parent', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='Launch_item_parent', to='production.Launch_items', verbose_name='Товарная позиция')),
            ],
            options={
                'verbose_name': 'Строка состава изделия',
                'ordering': ['section'],
                'unique_together': {('parent', 'child')},
            },
        ),
    ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException, JsonWSResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.sales.models.precent import Precent
from kaf_pas.sales.models.precent_download_file import download_present_file
from kaf_pas.sales.models.precent_upload_file import DSResponse__Precent_UploadFile

from kaf_pas.sales.models.precent_view import Precent_view, Precent_viewManager


@JsonResponseWithException()
def Precent_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Precent_view.objects.
                select_related('status').
                get_range_rows1(
                request=request,
                function=Precent_viewManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)\

@JsonResponseWithException()
def Precent_Add(request):
    return JsonResponse(DSResponseAdd(data=Precent.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Precent_Update(request):
    return JsonResponse(DSResponseUpdate(data=Precent.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Precent_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Precent.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Precent_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Precent.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Precent_Info(request):
    return JsonResponse(DSResponse(request=request, data=Precent.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonWSResponseWithException()
def Precent_UploadFile(request):
    return JsonResponse(DSResponse__Precent_UploadFile(request).response)


@JsonWSResponseWithException(printing=False)
def Precent_DownloadFile(request):
    return download_present_file(request)



# Generated by Django 3.0.3 on 2020-02-17 13:23

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0093_auto_20200217_0915'),
    ]

    operations = [
        migrations.AddField(
            model_name='operation_item',
            name='item_full_name',
            field=models.TextField(default=None),
        ),
        migrations.AddField(
            model_name='operation_item',
            name='item_full_name_obj',
            field=django.contrib.postgres.fields.jsonb.JSONField(default=dict),
        ),
    ]



from django.urls import path

from kaf_pas.accounting.views import tmp_buffer

urlpatterns = [

    path('TmpBuffers/Fetch/', tmp_buffer.Tmp_buffer_Fetch),
    path('TmpBuffers/Add', tmp_buffer.Tmp_buffer_Add),
    path('TmpBuffers/Update', tmp_buffer.Tmp_buffer_Update),
    path('TmpBuffers/Remove', tmp_buffer.Tmp_buffer_Remove),
    path('TmpBuffers/Lookup/', tmp_buffer.Tmp_buffer_Lookup),
    path('TmpBuffers/Info/', tmp_buffer.Tmp_buffer_Info),
    path('TmpBuffers/Copy', tmp_buffer.Tmp_buffer_Copy),

]



# Generated by Django 2.2 on 2019-04-26 14:01

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0014_operations_plan_refs'),
    ]

    operations = [
        migrations.CreateModel(
            name='Operations_plan_view',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('startdate', models.DateTimeField(db_index=True, default=None, verbose_name='Дата начала')),
                ('isFolder', models.BooleanField()),
            ],
            options={
                'verbose_name': 'Операции планировая',
                'db_table': 'production_operationsplan_view',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='Temp_production_operationsplan',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('qty', models.PositiveIntegerField()),
            ],
            options={
                'verbose_name': 'Временная таблица',
                'db_table': 'temp_production_operationsplan',
            },
        ),
        migrations.DeleteModel(
            name='TempProductionOperationsplan',
        ),
        migrations.AddField(
            model_name='operation_plan_templates',
            name='code',
            field=isc_common.fields.code_field.CodeField(unique=True),
        ),
        migrations.AddField(
            model_name='operation_plan_templates',
            name='description',
            field=isc_common.fields.description_field.DescriptionField(),
        ),
        migrations.AddField(
            model_name='operation_plan_templates',
            name='name',
            field=isc_common.fields.name_field.NameField(),
        ),
        migrations.AlterField(
            model_name='operationsplan',
            name='startdate',
            field=models.DateTimeField(db_index=True, default=None, verbose_name='Дата начала'),
        ),
        migrations.AddField(
            model_name='temp_production_operationsplan',
            name='operation_plan_templates',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='production.Operation_plan_templates'),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-28 12:11

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0112_auto_20200224_1448'),
    ]

    operations = [
        migrations.AddField(
            model_name='status_operation_types',
            name='color',
            field=isc_common.fields.code_field.CodeField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-28 21:08

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0100_auto_20191028_2105'),
    ]

    operations = [
        migrations.AddField(
            model_name='customer',
            name='props',
            field=bitfield.models.BitField((('real', 'Реальный клиент'),), db_index=True, default=1),
        ),
        migrations.AddField(
            model_name='dogovor_types',
            name='props',
            field=bitfield.models.BitField((('real', 'Реальный тим'),), db_index=True, default=1),
        ),
        migrations.AddField(
            model_name='statusdogovor',
            name='props',
            field=bitfield.models.BitField((('real', 'Реальный статус'),), db_index=True, default=1),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-18 15:05

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('ckk', '0210_auto_20200212_0752'),
        ('production', '0195_remove_launches_facets'),
        ('isc_common', '0050_deleted_progresses'),
        ('planing', '0097_auto_20200218_1218'),
    ]

    operations = [
        migrations.AddField(
            model_name='operation_operation',
            name='num',
            field=models.PositiveIntegerField(default=None),
        ),
        migrations.AddField(
            model_name='operations_view',
            name='color',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='isc_common.Standard_colors'),
        ),
        migrations.AddField(
            model_name='operations_view',
            name='creator',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='operations_view',
            name='date',
            field=models.DateTimeField(default=None),
        ),
        migrations.AddField(
            model_name='operations_view',
            name='edizm',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Ed_izm'),
        ),
        migrations.AddField(
            model_name='operations_view',
            name='isFolder',
            field=models.BooleanField(default=None),
        ),
        migrations.AddField(
            model_name='operations_view',
            name='item',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Item'),
        ),
        migrations.AddField(
            model_name='operations_view',
            name='item_full_name',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='operations_view',
            name='launch',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='production.Launches'),
        ),
        migrations.AddField(
            model_name='operations_view',
            name='location',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Locations'),
        ),
        migrations.AddField(
            model_name='operations_view',
            name='operation_level',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='planing.Levels'),
        ),
        migrations.AddField(
            model_name='operations_view',
            name='opertype',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='planing.Operation_types'),
        ),
        migrations.AddField(
            model_name='operations_view',
            name='production_operation',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='production.Operations'),
        ),
        migrations.AddField(
            model_name='operations_view',
            name='resource',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='production.Resource'),
        ),
        migrations.AddField(
            model_name='operations_view',
            name='status',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='planing.Status_operation_types'),
        ),
        migrations.AlterModelTable(
            name='operations_view',
            table=None,
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-07 18:02

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0038_auto_20191207_1801'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operation_launch_operation_material',
            name='edizm',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Ed_izm'),
        ),
        migrations.AlterField(
            model_name='operation_launch_operations_item',
            name='ed_izm',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Ed_izm'),
        ),
        migrations.AlterField(
            model_name='operation_launch_operations_item',
            name='num',
            field=models.PositiveIntegerField(),
        ),
        migrations.AlterField(
            model_name='operation_launch_operations_item',
            name='operation',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='planing.Operations'),
        ),
        migrations.AlterField(
            model_name='operation_launch_operations_item',
            name='qty',
            field=models.PositiveIntegerField(),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-28 12:40

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0113_status_operation_types_color'),
    ]

    operations = [
        migrations.AlterField(
            model_name='status_operation_types',
            name='props',
            field=bitfield.models.BitField((('disabled', 'Неактивная запись в гриде'), ('blink', 'Мигающее поле в гриде')), db_index=True, default=0),
        ),
    ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.sales.models.status_dogovor import StatusDogovor, StatusDogovorManager


@JsonResponseWithException()
def StatusDogovor_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=StatusDogovor.objects.
                exclude(code='virtual').
                get_range_rows1(
                request=request,
                function=StatusDogovorManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def StatusDogovor_Add(request):
    return JsonResponse(DSResponseAdd(data=StatusDogovor.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def StatusDogovor_Update(request):
    return JsonResponse(DSResponseUpdate(data=StatusDogovor.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def StatusDogovor_Remove(request):
    return JsonResponse(DSResponse(request=request, data=StatusDogovor.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def StatusDogovor_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=StatusDogovor.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def StatusDogovor_Info(request):
    return JsonResponse(DSResponse(request=request, data=StatusDogovor.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def StatusDogovor_Copy(request):
    return JsonResponse(DSResponse(request=request, data=StatusDogovor.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)            



import logging

from django.db.models import TextField, UniqueConstraint, Q

from crypto.models.crypto_file import Crypto_file, CryptoManager
from isc_common.fields.related import ForeignKeyProtect
from kaf_pas.kd.models.documents import Documents
from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy

logger = logging.getLogger(__name__)


class Documents_thumb10(Crypto_file):
    # Менять на cascade нельзя, потому как не происходит удаленеи файлов изображений при удалении документа
    document = ForeignKeyProtect(Documents, verbose_name='КД', null=True, blank=True)
    lotsman_document = ForeignKeyProtect(Lotsman_documents_hierarcy, verbose_name='Лоцман', null=True, blank=True)
    path = TextField()

    objects = CryptoManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'JPEG варианты документов'
        constraints = [
            UniqueConstraint(fields=['path'], condition=Q(document=None) & Q(lotsman_document=None), name='Documents_thumb10_unique_constraint_0'),
            UniqueConstraint(fields=['lotsman_document', 'path'], condition=Q(document=None), name='Documents_thumb10_unique_constraint_1'),
            UniqueConstraint(fields=['document', 'path'], condition=Q(lotsman_document=None), name='Documents_thumb10_unique_constraint_2'),
            UniqueConstraint(fields=['document', 'lotsman_document', 'path'], name='Documents_thumb10_unique_constraint_3'),
        ]



# Generated by Django 2.2.8 on 2019-12-07 18:20

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0041_auto_20191207_1811'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='resource_loading',
            name='launch_operations_item',
        ),
        migrations.RemoveField(
            model_name='resource_loading',
            name='resource',
        ),
        migrations.DeleteModel(
            name='Location_loading',
        ),
        migrations.DeleteModel(
            name='Resource_loading',
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-08 14:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0133_auto_20200208_1418'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='documents_thumb10',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='documents_thumb10',
            constraint=models.UniqueConstraint(condition=models.Q(('document', None), ('lotsman_document', None)), fields=('path',), name='Documents_thumb10_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='documents_thumb10',
            constraint=models.UniqueConstraint(condition=models.Q(document=None), fields=('lotsman_document', 'path'), name='Documents_thumb10_unique_constraint_1'),
        ),
        migrations.AddConstraint(
            model_name='documents_thumb10',
            constraint=models.UniqueConstraint(condition=models.Q(lotsman_document=None), fields=('document', 'path'), name='Documents_thumb10_unique_constraint_2'),
        ),
        migrations.AddConstraint(
            model_name='documents_thumb10',
            constraint=models.UniqueConstraint(fields=('document', 'lotsman_document', 'path'), name='Documents_thumb10_unique_constraint_3'),
        ),
    ]



from django.urls import path

from kaf_pas.ckk.views import item_location

urlpatterns = [

    path('Item_location/Fetch/', item_location.Item_location_Fetch),
    path('Item_location/Add', item_location.Item_location_Add),
    path('Item_location/Update', item_location.Item_location_Update),
    path('Item_location/Remove', item_location.Item_location_Remove),
    path('Item_location/Lookup/', item_location.Item_location_Lookup),
    path('Item_location/Info/', item_location.Item_location_Info),

]



# Generated by Django 2.2.8 on 2019-12-12 17:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0165_item_refs_hierarcy'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='item_refs_hierarcy',
            name='deleted_at',
        ),
        migrations.RemoveField(
            model_name='item_refs_hierarcy',
            name='deliting',
        ),
        migrations.RemoveField(
            model_name='item_refs_hierarcy',
            name='editing',
        ),
        migrations.RemoveField(
            model_name='item_refs_hierarcy',
            name='lastmodified',
        ),
        migrations.RemoveField(
            model_name='item_refs_hierarcy',
            name='parent',
        ),
        migrations.AlterField(
            model_name='item_refs_hierarcy',
            name='id',
            field=models.BigIntegerField(primary_key=True, serialize=False),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-28 22:05

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0044_auto_20190528_2155'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Precent_locations',
        ),
    ]



import logging

from django.db.models import BigIntegerField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditManager, AuditQuerySet
from isc_common.models.base_ref import Hierarcy
from isc_common.models.tree_audit import TreeAuditModelManager

from kaf_pas.ckk.models.item_refs import Item_refs

logger = logging.getLogger(__name__)


class Item_refs_hierarcyQuerySet(AuditQuerySet):
    pass


class Item_refs_hierarcyManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'parent_id': record.parent.id if record.parent else None,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Item_refs_hierarcyQuerySet(self.model, using=self._db)


class Item_refs_hierarcy(Hierarcy):
    id = BigIntegerField(primary_key=True, verbose_name="Идентификатор")

    item_refs = ForeignKeyProtect(Item_refs)

    objects = TreeAuditModelManager()

    def __str__(self):
        return f'\nID={self.id}, parent_id={self.parent_id}'

    class Meta:
        verbose_name = 'Item_refs_hierarcy'



# Generated by Django 2.2.2 on 2019-06-07 17:10

import bitfield.models
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0078_auto_20190606_1839'),
        ('kd', '0033_auto_20190606_1839'),
    ]

    operations = [
        migrations.CreateModel(
            name='Document_attributes',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('value_str', models.TextField(db_index=True, verbose_name='Значение атрибута')),
                ('props', bitfield.models.BitField((('relevant', 'Актуальность'), ('copied', 'Копированный')), db_index=True, default=1)),
                ('attr_type', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Attr_type', verbose_name='Тип атрибута')),
            ],
            options={
                'verbose_name': 'Аттрибуты докуменнта',
                'unique_together': {('attr_type', 'value_str')},
            },
        ),
        migrations.CreateModel(
            name='Document_attr_cross',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('attribute', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='kd.Document_attributes')),
                ('document', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='kd.Documents')),
            ],
            options={
                'verbose_name': 'Кросс таблица',
            },
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-07 18:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0080_auto_20190607_1852'),
        ('kd', '0034_document_attr_cross_document_attributes'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='document_attributes',
            unique_together={('attr_type', 'value_str', 'props')},
        ),
    ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException, JsonWSResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.production.models.launches import Launches
from kaf_pas.production.models.launches_view import Launches_view, Launches_viewManager


@JsonResponseWithException()
def Launches_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Launches_view.objects.
                select_related('demand', 'status').
                get_range_rows1(
                request=request,
                function=Launches_viewManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonWSResponseWithException()
def Launches_Add(request):
    return JsonResponse(DSResponseAdd(data=Launches.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonWSResponseWithException()
def Launches_Update(request):
    return JsonResponse(DSResponseUpdate(data=Launches.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)

@JsonWSResponseWithException()
def Launches_ReCreate(request):
    return JsonResponse(DSResponseUpdate(data=Launches.objects.reCreateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)

@JsonWSResponseWithException()
def Launches_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Launches.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Launches_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Launches.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Launches_Info(request):
    return JsonResponse(DSResponse(request=request, data=Launches.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Launches_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Launches.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 3.0.3 on 2020-02-19 03:16

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0210_auto_20200212_0752'),
        ('planing', '0098_auto_20200218_1505'),
    ]

    operations = [
        migrations.AddField(
            model_name='operation_operation',
            name='ed_izm',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Ed_izm'),
        ),
        migrations.AddField(
            model_name='operation_operation',
            name='qty',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='operation_operation',
            name='num',
            field=models.PositiveIntegerField(),
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-17 04:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0087_launch_item_line_launch_item_refs_launch_items_launches'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Item_refs',
        ),
    ]



from django.urls import path

from kaf_pas.kd.views import document_attributes_SPC_CLM_FORMAT

urlpatterns = [

    path('Document_attributes_SPC_CLM_FORMAT/Fetch/', document_attributes_SPC_CLM_FORMAT.Document_attributes_SPC_CLM_FORMAT_Fetch),
    path('Document_attributes_SPC_CLM_FORMAT/Add', document_attributes_SPC_CLM_FORMAT.Document_attributes_SPC_CLM_FORMAT_Add),
    path('Document_attributes_SPC_CLM_FORMAT/Update', document_attributes_SPC_CLM_FORMAT.Document_attributes_SPC_CLM_FORMAT_Update),
    path('Document_attributes_SPC_CLM_FORMAT/Remove', document_attributes_SPC_CLM_FORMAT.Document_attributes_SPC_CLM_FORMAT_Remove),
    path('Document_attributes_SPC_CLM_FORMAT/Lookup/', document_attributes_SPC_CLM_FORMAT.Document_attributes_SPC_CLM_FORMAT_Lookup),
    path('Document_attributes_SPC_CLM_FORMAT/Info/', document_attributes_SPC_CLM_FORMAT.Document_attributes_SPC_CLM_FORMAT_Info),
    path('Document_attributes_SPC_CLM_FORMAT/Copy', document_attributes_SPC_CLM_FORMAT.Document_attributes_SPC_CLM_FORMAT_Copy),

]



# Generated by Django 2.2.7 on 2019-11-20 14:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0159_auto_20191119_2034'),
        ('production', '0126_auto_20191120_1441'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='launch_item_line',
            unique_together={('parent', 'child', 'launch', 'added_parent')},
        ),
    ]



import logging

from django.contrib.postgres.fields import JSONField
from django.db.models import TextField

from isc_common.fields.related import ForeignKeyCascade
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from kaf_pas.production.models.ready_2_launch import Ready_2_launch

logger = logging.getLogger(__name__)


class Ready_2_launch_detailQuerySet(AuditQuerySet):
    def delete(self):
        super().delete()


class Ready_2_launch_detailManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'item_full_name': record.item_full_name,
            'notes': record.notes,
            'ready_id': record.ready.id,
            'ready__lastmodified': record.ready.lastmodified,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Ready_2_launch_detailQuerySet(self.model, using=self._db)


class Ready_2_launch_detail(AuditModel):
    ready = ForeignKeyCascade(Ready_2_launch)
    item_full_name = TextField(null=True, blank=True)
    item_full_name_obj = JSONField(default=dict)
    notes = TextField(null=True, blank=True)

    objects = Ready_2_launch_detailManager()

    def __str__(self):
        return f"ID:{self.id}"

    class Meta:
        verbose_name = 'Детализация готовности к запуску'



import logging

from django.db import ProgrammingError
from django.forms import model_to_dict

logger = logging.getLogger(__name__)


class Operations_Plan:
    def get_first_item_of_tuple(self, tp):
        res, _ = tp
        return res

    def __init__(self):
        from kaf_pas.planing.models.operation_types import Operation_types
        from kaf_pas.planing.models.operation_types import Operation_typesManager

        try:
            self.OPERSTYPES = [Operation_typesManager.getRecord(opertype) for opertype in Operation_types.objects.all()]

        except ProgrammingError as ex:
            logger.warning(ex)



from django.db import transaction

from isc_common.http.DSRequest import DSRequest
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.ckk.models.item import Item
from kaf_pas.production.models.launch_operation_material import Launch_operations_material
from kaf_pas.production.models.launch_operations_item import Launch_operations_item, Launch_operations_itemManager


@JsonResponseWithException()
def Launch_operations_item_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Launch_operations_item.objects.
                filter().
                get_range_rows1(
                request=request,
                function=Launch_operations_itemManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Launch_operations_item_Add(request):
    return JsonResponse(DSResponseAdd(data=Launch_operations_item.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Launch_operations_item_Update(request):
    return JsonResponse(DSResponseUpdate(data=Launch_operations_item.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Launch_operations_item_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Launch_operations_item.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Launch_operations_item_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Launch_operations_item.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Launch_operations_item_Info(request):
    return JsonResponse(DSResponse(request=request, data=Launch_operations_item.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Launch_operations_item_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Launch_operations_item.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)

@JsonResponseWithException()
def Launch_operations_item_CopyOpers(request):
    _request = DSRequest(request=request)
    source = _request.json.get('source')
    destination = _request.json.get('destination')
    res = False

    if source and destination:
        srecords = source.get('records')
        drecord = destination.get('record')
        item = Item.objects.get(id=drecord.get('id'))

        if isinstance(srecords, list) and isinstance(drecord, dict):
            with transaction.atomic():
                for srecord in srecords:
                    old_launch_operations_item = Launch_operations_item.objects.get(id=srecord.get('id'))
                    launch_operations_item, created = Launch_operations_item.objects.get_or_create(
                        item=item,
                        operation=old_launch_operations_item.operation,
                        defaults=dict(
                            ed_izm=old_launch_operations_item.ed_izm,
                            operationitem=old_launch_operations_item.operationitem,
                            qty=old_launch_operations_item.qty,
                            num=old_launch_operations_item.num,
                            description=old_launch_operations_item.description,
                        ))

                    if created:
                        for launch_operation_resources in launch_operation_resources.objects.filter(launch_operationitem=old_launch_operations_item):
                            launch_operation_resources.objects.get_or_create(
                                launch_operationitem=launch_operations_item,
                                operation_resources=launch_operation_resources.operation_resources,
                                resource=launch_operation_resources.resource,
                                location=launch_operation_resources.location
                            )

                        for launch_operation_material in Launch_operations_material.objects.filter(launch_operationitem=old_launch_operations_item):
                            Launch_operations_material.objects.get_or_create(
                                launch_operationitem=launch_operations_item,
                                operation_material=launch_operations_item.operation_material,
                                material=launch_operation_material.material,
                                material_askon=launch_operation_material.material_askon,
                                ed_izm=launch_operation_material.ed_izm,
                                qty=launch_operation_material.qty,
                            )

        return JsonResponse(DSResponse(request=request, status=RPCResponseConstant.statusSuccess).response)
    else:
        raise Exception(f'Копирование не выполнено.')



# Generated by Django 2.2 on 2019-04-17 18:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0020_cdw_attrs_spw_attrs'),
        ('ckk', '0027_auto_20190417_1821'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='item_image_refs',
            unique_together={('item', 'thumb', 'thumb10')},
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-22 12:31

import django.db.models.deletion
from django.db import migrations

import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0076_precent_items_itemtype'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='demand',
            name='customer',
        ),
        migrations.AddField(
            model_name='demand',
            name='document',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='sales.Dogovors'),
        ),
        migrations.AddField(
            model_name='demand',
            name='precent',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='sales.Precent'),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-28 15:34

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0030_auto_20190525_1331'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='documents_view',
            table='kd_documents_mview',
        ),
        migrations.AlterModelTable(
            name='pathes_view',
            table='kd_pathes_mview',
        ),
    ]



import logging

from django.core.management import BaseCommand
from django.db import transaction

from kaf_pas.ckk.models.tmp_item_operations_view import Tmp_Item_operations_viewManager, Tmp_Item_operations_view
from kaf_pas.ckk.models.tmp_item_refs import Tmp_Item_refsManager
from kaf_pas.ckk.views.item_view import UploadTmpTable

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Тестирование"

    def handle(self, *args, **options):
        logger.info(self.help)

        with transaction.atomic():
            Tmp_Item_refsManager.create()
            Tmp_Item_operations_viewManager.create()

            UploadTmpTable(dict(id=523)).make_tmp()
            print(f'Cnt: {Tmp_Item_operations_view.objects.count()}')



# Generated by Django 3.0.3 on 2020-02-08 14:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0131_auto_20200207_0822'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='document_attributes',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='document_attributes',
            constraint=models.UniqueConstraint(condition=models.Q(value_int=None), fields=('attr_type', 'value_str'), name='Document_attributes_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='document_attributes',
            constraint=models.UniqueConstraint(fields=('attr_type', 'value_int', 'value_str'), name='Document_attributes_unique_constraint_1'),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-15 01:13

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0110_auto_20191215_0108'),
    ]

    operations = [
        migrations.AlterField(
            model_name='lotsman_documents_hierarcy_refs',
            name='parent',
            field=isc_common.fields.related.ForeignKeyCascade(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='Lotsman_documents_hierarcy_parent', to='kd.Lotsman_documents_hierarcy'),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-04 12:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0077_documents_mview'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='documents_thumb',
            name='props',
        ),
        migrations.RemoveField(
            model_name='documents_thumb10',
            name='props',
        ),
    ]



import logging

from bitfield import BitField
from django.db.models import CheckConstraint, Q, F, UniqueConstraint

from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditModel
from isc_common.models.tree_audit import TreeAuditModelManager, TreeAuditModelQuerySet
from kaf_pas.planing.models.operations import Operations

logger = logging.getLogger(__name__)


class Operation_refsQuerySet(TreeAuditModelQuerySet):
    pass


class Operation_refsManager(TreeAuditModelManager):

    @staticmethod
    def props():
        return BitField(flags=(
            ('inner_routing', 'Связь операций внутри товарной позиции'),  # 1
            ('outer_routing', 'Связь операций между товарными позициями'),  # 2
            ('product_order_routing', 'Связи в блоке заказы на производство'),  # 4
        ), default=0, db_index=True)

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'parent': record.parent.id if record.parent else None,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Operation_refsQuerySet(self.model, using=self._db)


class Operation_refs(AuditModel):
    parent = ForeignKeyProtect(Operations, related_name='operation_parent', blank=True, null=True)
    parent_real = ForeignKeyProtect(Operations, related_name='operation_parent_real', blank=True, null=True)
    child = ForeignKeyProtect(Operations, related_name='operation_child')
    props = Operation_refsManager.props()

    objects = Operation_refsManager()

    def __str__(self):
        return f"ID:{self.id}, child: [{self.child}] , parent: [{self.parent}]"

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Кросс таблица операций планирования'
        unique_together = ('child', 'parent', 'props')
        constraints = [
            CheckConstraint(check=~Q(child=F('parent')), name='Operation_refs_not_circulate_refs'),
            UniqueConstraint(fields=['child'], condition=Q(parent=None) & Q(parent_real=None), name='Operation_refs_unique_constraint_0'),
            UniqueConstraint(fields=['child', 'parent'], condition=Q(parent_real=None), name='Operation_refs_unique_constraint_1'),
            UniqueConstraint(fields=['child', 'parent_real'], condition=Q(parent=None), name='Operation_refs_unique_constraint_2'),
            UniqueConstraint(fields=['child', 'parent', 'parent_real'], name='Operation_refs_unique_constraint_3'),
        ]



# Generated by Django 2.2.1 on 2019-05-21 18:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0051_remove_status_id_owner'),
        ('ckk', '0043_auto_20190521_1853'),
    ]

    operations = [
    ]



# Generated by Django 2.2.8 on 2019-12-06 15:53

import bitfield.models
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0028_auto_20191206_1329'),
    ]

    operations = [
        migrations.CreateModel(
            name='Status_operation_types',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('code', isc_common.fields.code_field.CodeField(blank=True, null=True)),
                ('name', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('description', isc_common.fields.description_field.DescriptionField()),
                ('props', bitfield.models.BitField(('disabled', 'Неактивная запись в гриде'), db_index=True, default=0)),
                ('opertype', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='planing.Operation_types')),
            ],
            options={
                'verbose_name': 'Статусы запусков',
            },
        ),
    ]



from django.urls import path

from kaf_pas.production.views import operation_attr

urlpatterns = [

    path('Operation_attr/Fetch/', operation_attr.Operation_attr_Fetch),
    path('Operation_attr/Add', operation_attr.Operation_attr_Add),
    path('Operation_attr/Update', operation_attr.Operation_attr_Update),
    path('Operation_attr/Remove', operation_attr.Operation_attr_Remove),
    path('Operation_attr/Lookup/', operation_attr.Operation_attr_Lookup),
    path('Operation_attr/Info/', operation_attr.Operation_attr_Info),
    path('Operation_attr/Copy', operation_attr.Operation_attr_Copy),

]



import logging

from django.core.management import BaseCommand

from kaf_pas.production.models.operations_item import Operations_itemManager

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Тестирование"

    def handle(self, *args, **options):
        Operations_itemManager.refresh_num1()



from django.urls import path

from kaf_pas.kd.views import documents

urlpatterns = [

    path('Documents/Fetch/', documents.Documents_Fetch),
    path('Documents/Add', documents.Documents_Add),
    path('Documents/Update', documents.Documents_Update),
    path('Documents/Remove', documents.Documents_Remove),
    path('Documents/Lookup/', documents.Documents_Lookup),
    path('Documents/Info/', documents.Documents_Info),
    path('Documents/MakeItem', documents.Documents_MakeItem),
    path('Documents/Documents_Treat', documents.Documents_Treat),
    path('Documents/ReloadDoc', documents.Documents_ReloadDoc),
]
